// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

/// iOS side implementation of [OperationsPlatformInterface].
class MethodChannelIosOperations extends OperationsPlatformInterface {
  final MethodChannel _channel;

  /// Default constructor for [MethodChannelIosOperations].
  MethodChannelIosOperations(this._channel);

  @override
  Future<void> outOfBandOperation({
    required String operationId,
    required String subOperationId,
    OutOfBandPayload? payload,
    Function? onRegistration,
    Function? onAuthentication,
    RequestHeaders? requestHeaders,
    required Function(OutOfBandPlatformOperationType) onOperationType,
    Function(OutOfBandOperationError)? onError,
  }) async {
    final operation = OutOfBandPlatformOperation(
      operationId: operationId,
      onOperationType: onOperationType,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = OobOperationMessage(
      operationId: operationId,
      subOperationId: subOperationId,
      payload: payload,
      requestHeaders: requestHeaders,
      onRegistrationProvided: (onRegistration != null),
      onAuthenticationProvided: (onAuthentication != null),
      onSuccessProvided: false,
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.oobOperation,
      message,
    );
  }

  @override
  Future<void> outOfBandRegister({
    required String operationId,
    DeviceInformation? deviceInformation,
    bool? allowClass2AndroidSensors,
    bool? allowDevicePasscodeAsFallback,
    bool? invalidateOnNewOsBiometrics,
    RequestHeaders? requestHeaders,
    AuthenticatorSelector? authenticatorSelector,
    PinEnroller? pinEnroller,
    BiometricUserVerifier? biometricUserVerifier,
    DevicePasscodeUserVerifier? devicePasscodeUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    Function? onSuccess,
    Function(OperationError)? onError,
  }) async {
    final registration = OutOfBandRegistrationPlatformOperation(
      operationId: operationId,
      authenticatorSelector: authenticatorSelector,
      pinEnroller: pinEnroller,
      biometricUserVerifier: biometricUserVerifier,
      devicePasscodeUserVerifier: devicePasscodeUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(registration);
    Map<String, dynamic> message = OobRegistrationMessage(
      operationId: operationId,
      deviceInformation: deviceInformation,
      pinPolicy: pinEnroller?.pinPolicy,
      allowClass2AndroidSensors: allowClass2AndroidSensors,
      allowDevicePasscodeAsFallback: allowDevicePasscodeAsFallback,
      invalidateOnNewOsBiometrics: invalidateOnNewOsBiometrics,
      requestHeaders: requestHeaders,
      accountSelectorProvided: false,
      authenticatorSelectorProvided: (authenticatorSelector != null),
      pinEnrollerProvided: (pinEnroller != null),
      pinUserVerifierProvided: false,
      biometricUserVerifierProvided: (biometricUserVerifier != null),
      devicePasscodeUserVerifierProvided: (devicePasscodeUserVerifier != null),
      fingerprintUserVerifierProvided: (fingerprintUserVerifier != null),
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.oobRegister,
      message,
    );
  }

  @override
  Future<void> outOfBandAuthenticate({
    required String operationId,
    RequestHeaders? requestHeaders,
    AccountSelector? accountSelector,
    AuthenticatorSelector? authenticatorSelector,
    PinUserVerifier? pinUserVerifier,
    BiometricUserVerifier? biometricUserVerifier,
    DevicePasscodeUserVerifier? devicePasscodeUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    Function(AuthorizationProvider?)? onSuccess,
    Function(OperationError)? onError,
  }) async {
    final authentication = OutOfBandAuthenticationPlatformOperation(
      operationId: operationId,
      accountSelector: accountSelector,
      authenticatorSelector: authenticatorSelector,
      pinUserVerifier: pinUserVerifier,
      biometricUserVerifier: biometricUserVerifier,
      devicePasscodeUserVerifier: devicePasscodeUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(authentication);
    Map<String, dynamic> message = OobAuthenticationMessage(
      operationId: operationId,
      requestHeaders: requestHeaders,
      accountSelectorProvided: (accountSelector != null),
      authenticatorSelectorProvided: (authenticatorSelector != null),
      pinEnrollerProvided: false,
      pinUserVerifierProvided: (pinUserVerifier != null),
      biometricUserVerifierProvided: (biometricUserVerifier != null),
      devicePasscodeUserVerifierProvided: (devicePasscodeUserVerifier != null),
      fingerprintUserVerifierProvided: (fingerprintUserVerifier != null),
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.oobAuthenticate,
      message,
    );
  }

  @override
  Future<void> outOfBandPayloadDecode({
    required String operationId,
    String? json,
    String? base64UrlEncoded,
    Function(OutOfBandPayload?)? onSuccess,
    Function(OutOfBandPayloadError)? onError,
  }) async {
    final operation = OutOfBandPayloadDecodePlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = OobPayloadDecodeMessage(
      operationId: operationId,
      json: json,
      base64UrlEncoded: base64UrlEncoded,
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.payloadDecode,
      message,
    );
  }

  @override
  Future<void> register({
    required String operationId,
    String? username,
    String? serverUrl,
    DeviceInformation? deviceInformation,
    AuthorizationProvider? authorizationProvider,
    bool? allowClass2AndroidSensors,
    bool? allowDevicePasscodeAsFallback,
    bool? invalidateOnNewOsBiometrics,
    RequestHeaders? requestHeaders,
    AuthenticatorSelector? authenticatorSelector,
    PinEnroller? pinEnroller,
    BiometricUserVerifier? biometricUserVerifier,
    DevicePasscodeUserVerifier? devicePasscodeUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    Function? onSuccess,
    Function(OperationError)? onError,
  }) async {
    final operation = RegistrationPlatformOperation(
      operationId: operationId,
      authenticatorSelector: authenticatorSelector,
      pinEnroller: pinEnroller,
      biometricUserVerifier: biometricUserVerifier,
      devicePasscodeUserVerifier: devicePasscodeUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = RegistrationMessage(
      operationId: operationId,
      username: username,
      serverUrl: serverUrl,
      deviceInformation: deviceInformation,
      authorizationProvider: authorizationProvider,
      pinPolicy: pinEnroller?.pinPolicy,
      allowClass2AndroidSensors: allowClass2AndroidSensors,
      allowDevicePasscodeAsFallback: allowDevicePasscodeAsFallback,
      invalidateOnNewOsBiometrics: invalidateOnNewOsBiometrics,
      requestHeaders: requestHeaders,
      accountSelectorProvided: false,
      authenticatorSelectorProvided: (authenticatorSelector != null),
      pinEnrollerProvided: (pinEnroller != null),
      pinUserVerifierProvided: false,
      biometricUserVerifierProvided: (biometricUserVerifier != null),
      devicePasscodeUserVerifierProvided: (devicePasscodeUserVerifier != null),
      fingerprintUserVerifierProvided: (fingerprintUserVerifier != null),
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.register,
      message,
    );
  }

  @override
  Future<void> authCloudApiRegister({
    required String operationId,
    String? enrollResponse,
    String? appLinkUri,
    DeviceInformation? deviceInformation,
    bool? allowClass2AndroidSensors,
    bool? allowDevicePasscodeAsFallback,
    bool? invalidateOnNewOsBiometrics,
    RequestHeaders? requestHeaders,
    AuthenticatorSelector? authenticatorSelector,
    PinEnroller? pinEnroller,
    BiometricUserVerifier? biometricUserVerifier,
    DevicePasscodeUserVerifier? devicePasscodeUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    Function? onSuccess,
    Function(AuthCloudApiError)? onError,
  }) async {
    final operation = AuthCloudApiRegistrationPlatformOperation(
      operationId: operationId,
      authenticatorSelector: authenticatorSelector,
      pinEnroller: pinEnroller,
      biometricUserVerifier: biometricUserVerifier,
      devicePasscodeUserVerifier: devicePasscodeUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = AuthCloudApiRegistrationMessage(
      operationId: operationId,
      enrollResponse: enrollResponse,
      appLinkUri: appLinkUri,
      deviceInformation: deviceInformation,
      pinPolicy: pinEnroller?.pinPolicy,
      allowClass2AndroidSensors: allowClass2AndroidSensors,
      allowDevicePasscodeAsFallback: allowDevicePasscodeAsFallback,
      invalidateOnNewOsBiometrics: invalidateOnNewOsBiometrics,
      requestHeaders: requestHeaders,
      accountSelectorProvided: false,
      authenticatorSelectorProvided: (authenticatorSelector != null),
      pinEnrollerProvided: (pinEnroller != null),
      pinUserVerifierProvided: false,
      biometricUserVerifierProvided: (biometricUserVerifier != null),
      devicePasscodeUserVerifierProvided: (devicePasscodeUserVerifier != null),
      fingerprintUserVerifierProvided: (fingerprintUserVerifier != null),
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.authCloudApiRegister,
      message,
    );
  }

  @override
  Future<void> authenticate({
    required String operationId,
    String? username,
    SessionProvider? sessionProvider,
    RetryPolicy? retryPolicy,
    RequestHeaders? requestHeaders,
    AuthenticatorSelector? authenticatorSelector,
    PinUserVerifier? pinUserVerifier,
    BiometricUserVerifier? biometricUserVerifier,
    DevicePasscodeUserVerifier? devicePasscodeUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    Function(AuthorizationProvider?)? onSuccess,
    Function(AuthenticationError)? onError,
  }) async {
    final operation = AuthenticationPlatformOperation(
        operationId: operationId,
        authenticatorSelector: authenticatorSelector,
        biometricUserVerifier: biometricUserVerifier,
        devicePasscodeUserVerifier: devicePasscodeUserVerifier,
        fingerprintUserVerifier: fingerprintUserVerifier,
        pinUserVerifier: pinUserVerifier,
        onSuccessCallback: onSuccess,
        onErrorCallback: onError);
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = AuthenticationMessage(
      operationId: operationId,
      username: username,
      sessionProvider: sessionProvider,
      retryPolicy: retryPolicy,
      requestHeaders: requestHeaders,
      accountSelectorProvided: false,
      authenticatorSelectorProvided: (authenticatorSelector != null),
      pinEnrollerProvided: false,
      pinUserVerifierProvided: (pinUserVerifier != null),
      biometricUserVerifierProvided: (biometricUserVerifier != null),
      devicePasscodeUserVerifierProvided: (devicePasscodeUserVerifier != null),
      fingerprintUserVerifierProvided: (fingerprintUserVerifier != null),
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.authenticate,
      message,
    );
  }

  @override
  Future<void> deregister({
    required String operationId,
    String? username,
    String? aaid,
    AuthorizationProvider? authorizationProvider,
    RequestHeaders? requestHeaders,
    Function? onSuccess,
    Function(OperationError)? onError,
  }) async {
    final operation = DeregistrationPlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = DeregistrationMessage(
      operationId: operationId,
      username: username,
      aaid: aaid,
      authorizationProvider: authorizationProvider,
      requestHeaders: requestHeaders,
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.deregister,
      message,
    );
  }

  @override
  Future<void> changePin({
    required String operationId,
    String? username,
    PinChanger? pinChanger,
    Function? onSuccess,
    Function(PinChangeError)? onError,
  }) async {
    final operation = ChangePinPlatformOperation(
      operationId: operationId,
      pinChanger: pinChanger,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = ChangePinMessage(
      operationId: operationId,
      username: username,
      pinPolicy: pinChanger?.pinPolicy,
      pinChangerProvided: (pinChanger != null),
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.changePin,
      message,
    );
  }

  @override
  Future<void> deviceInformationChange({
    required String operationId,
    String? name,
    String? fcmRegistrationToken,
    bool? disablePushNotifications,
    RetryPolicy? retryPolicy,
    RequestHeaders? requestHeaders,
    Function? onSuccess,
    Function(DeviceInformationChangeError)? onError,
  }) async {
    final operation = DeviceInformationChangePlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = DeviceInformationChangeMessage(
      operationId: operationId,
      name: name,
      fcmRegistrationToken: fcmRegistrationToken,
      disablePushNotifications: disablePushNotifications,
      retryPolicy: retryPolicy,
      requestHeaders: requestHeaders,
      onSuccessProvided: (onSuccess != null),
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.deviceInformationChange,
      message,
    );
  }
}
