// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';

import '../mocks/method_channel_mock.dart';
import '../test_data/common_test_data.dart';
import '../test_data/device_information_test_data.dart';
import '../test_data/local_data_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    final sdk = NevisMobileAuthenticationSdkIos();

    group("Given accounts is called", () {
      group("When no account is registered", () {
        test("Then number of registered accounts is 0", () async {
          MethodChannelMock(
            method: 'localAccounts',
            result: LocalDataTestData.accountsEmptyResponse,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.accounts(operationId);
          expect(result.length, 0);
        });
      });

      group("When one account is registered", () {
        test("Then number of registered accounts is 1", () async {
          MethodChannelMock(
            method: 'localAccounts',
            result: LocalDataTestData.accountsOneAccountResponse,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.accounts(operationId);
          expect(result.length, 1);
          expect(result.first.username, CommonTestData.username1);
        });
      });

      group("When two accounts is registered", () {
        test("Then number of registered accounts is 2", () async {
          MethodChannelMock(
            method: 'localAccounts',
            result: LocalDataTestData.accountsTwoAccountsResponse,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.accounts(operationId);
          expect(result.length, 2);
          expect(result.first.username, CommonTestData.username1);
          expect(result.last.username, CommonTestData.username2);
        });
      });
    });

    group("Given authenticators is called", () {
      group("When no authenticator is available", () {
        test("Then number of authenticators is 0", () async {
          MethodChannelMock(
            method: 'localAuthenticators',
            result: LocalDataTestData.authenticatorsEmptyResponse,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.authenticators(operationId);
          expect(result.length, 0);
        });
      });

      group("When one authenticator is available", () {
        test("Then number of authenticators is 1", () async {
          MethodChannelMock(
            method: 'localAuthenticators',
            result: LocalDataTestData.authenticatorsOneAuthenticatorResponse,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.authenticators(operationId);
          expect(result.length, 1);
          expect(result.first.aaid, CommonTestData.aaid1);
        });
      });

      group("When two authenticators are available", () {
        test("Then number of authenticators is 2", () async {
          MethodChannelMock(
            method: 'localAuthenticators',
            result: LocalDataTestData.authenticatorsTwoAuthenticatorsResponse,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.authenticators(operationId);
          expect(result.length, 2);
          expect(result.first.aaid, CommonTestData.aaid1);
          expect(result.last.aaid, CommonTestData.aaid2);
        });
      });
    });

    group("Given deviceInformation is called", () {
      group("When device information exists", () {
        test("Then it returns the device information", () async {
          MethodChannelMock(
            method: 'localDeviceInformation',
            result: LocalDataTestData.deviceInformationResult,
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.deviceInformation(operationId);
          expect(result?.name, DeviceInformationTestData.deviceName);
          expect(
            result?.fcmRegistrationToken,
            DeviceInformationTestData.fcmRegistrationToken,
          );
        });
      });

      group("When device information does not exist", () {
        test("Then it returns null", () async {
          MethodChannelMock(
            method: 'localDeviceInformation',
            result: {},
          );
          final operationId = CommonTestData.operationId;
          final result = await sdk.localData.deviceInformation(operationId);
          expect(result, null);
        });
      });
    });

    group("Given deleteAuthenticator is called", () {
      group("When username and AAID are provided", () {
        test("Then authenticator is deleted for the specific user", () async {
          final channel = MethodChannelMock(
            method: 'localDeleteAuthenticator',
            result: CommonTestData.emptyResponse,
          );
          await sdk.localData.deleteAuthenticator(
            operationId: CommonTestData.operationId,
            username: CommonTestData.username1,
            aaid: CommonTestData.aaid1,
          );
          expect(
            channel.mappedArgs,
            LocalDataTestData.deleteAuthenticatorCorrectArguments,
          );
        });
      });
    });
  });
}
