// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/authentication_test_data.dart';
import '../test_data/authorization_provider_test_data.dart';
import '../test_data/common_test_data.dart';
import '../test_data/request_headers_test_data.dart';
import '../test_data/retry_policy_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache cache;
    late MethodChannelMock channel;
    const methodName = 'authenticate';

    group("Given $methodName is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        cache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = cache;
        channel = MethodChannelMock(
          method: methodName,
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        group("When the required parameters are correct", () {
          test("Then result is success", () async {
            final authenticatorSelector = MockAuthenticatorSelector();
            final pinUserVerifier = MockPinUserVerifier();
            final biometricUserVerifier = MockBiometricUserVerifier();
            final devicePasscodeUserVerifier = MockDevicePasscodeUserVerifier();
            final fingerprintUserVerifier = MockFingerprintUserVerifier();
            final operation = MockAuthenticationPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);

            await sdk.operations.authenticate(
                operationId: CommonTestData.operationId,
                username: CommonTestData.username1,
                authenticatorSelector: authenticatorSelector,
                pinUserVerifier: pinUserVerifier,
                biometricUserVerifier: biometricUserVerifier,
                devicePasscodeUserVerifier: devicePasscodeUserVerifier,
                fingerprintUserVerifier: fingerprintUserVerifier,
                onSuccess: (provider) {},
                onError: (error) {});

            expect(
              channel.mappedArgs,
              AuthenticationTestData.correctArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<AuthenticationPlatformOperation>(),
              )),
            );
          });
        });

        group("When all parameters are correct", () {
          test("Then result is success", () async {
            final sessionProvider = MockSessionProvider();
            final retryPolicy = MockRetryPolicy();
            final requestHeaders = MockRequestHeaders();
            final authenticatorSelector = MockAuthenticatorSelector();
            final pinUserVerifier = MockPinUserVerifier();
            final biometricUserVerifier = MockBiometricUserVerifier();
            final devicePasscodeUserVerifier = MockDevicePasscodeUserVerifier();
            final fingerprintUserVerifier = MockFingerprintUserVerifier();
            final operation = MockAuthenticationPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);
            when(sessionProvider.toJson()).thenReturn(
              AuthorizationProviderTestData.typedCookieSessionProviderMap,
            );
            when(requestHeaders.toJson()).thenReturn(
              RequestHeadersTestData.requestHeadersMap,
            );
            when(retryPolicy.toJson()).thenReturn(
              RetryPolicyTestData.constantRetryPolicyMap,
            );

            await sdk.operations.authenticate(
                operationId: CommonTestData.operationId,
                username: CommonTestData.username1,
                sessionProvider: sessionProvider,
                retryPolicy: retryPolicy,
                requestHeaders: requestHeaders,
                authenticatorSelector: authenticatorSelector,
                pinUserVerifier: pinUserVerifier,
                biometricUserVerifier: biometricUserVerifier,
                devicePasscodeUserVerifier: devicePasscodeUserVerifier,
                fingerprintUserVerifier: fingerprintUserVerifier,
                onSuccess: (provider) {},
                onError: (error) {});

            expect(
              channel.mappedArgs,
              AuthenticationTestData.correctAllArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<AuthenticationPlatformOperation>(),
              )),
            );
          });
        });
      });
    });
  });
}
