// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/device_information_change_test_data.dart';
import '../test_data/device_information_test_data.dart';
import '../test_data/request_headers_test_data.dart';
import '../test_data/retry_policy_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache cache;
    late MethodChannelMock channel;
    const methodName = 'deviceInformationChange';

    group("Given $methodName is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        cache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = cache;
        channel = MethodChannelMock(
          method: methodName,
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        final deviceName = DeviceInformationTestData.deviceName;
        final fcmToken = DeviceInformationTestData.fcmRegistrationToken;

        group("When the default parameters are given", () {
          test("Then result is success", () async {
            final operation = MockDeviceInformationChangePlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);

            await sdk.operations.deviceInformationChange(
              operationId: CommonTestData.operationId,
              name: deviceName,
              fcmRegistrationToken: fcmToken,
              disablePushNotifications: false,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              DeviceInformationChangeTestData.correctArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<DeviceInformationChangePlatformOperation>(),
              )),
            );
          });
        });

        group("When no retry policy is given", () {
          test("Then result is success", () async {
            final operation = MockDeviceInformationChangePlatformOperation();
            final retryPolicy = MockRetryPolicy();
            final requestHeaders = MockRequestHeaders();
            when(cache.createOperation(any)).thenReturn(operation);
            when(retryPolicy.toJson()).thenReturn(
              RetryPolicyTestData.noRetryPolicyMap,
            );
            when(requestHeaders.toJson()).thenReturn(
              RequestHeadersTestData.requestHeadersMap,
            );

            await sdk.operations.deviceInformationChange(
              operationId: CommonTestData.operationId,
              name: deviceName,
              fcmRegistrationToken: fcmToken,
              disablePushNotifications: false,
              retryPolicy: retryPolicy,
              requestHeaders: requestHeaders,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              DeviceInformationChangeTestData.correctNoRetryArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<DeviceInformationChangePlatformOperation>(),
              )),
            );
          });
        });

        group("When constant retry policy is given", () {
          test("Then result is success", () async {
            final operation = MockDeviceInformationChangePlatformOperation();
            final retryPolicy = MockRetryPolicy();
            when(cache.createOperation(any)).thenReturn(operation);
            when(retryPolicy.toJson()).thenReturn(
              RetryPolicyTestData.constantRetryPolicyMap,
            );

            await sdk.operations.deviceInformationChange(
              operationId: CommonTestData.operationId,
              name: deviceName,
              fcmRegistrationToken: fcmToken,
              disablePushNotifications: false,
              retryPolicy: retryPolicy,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              DeviceInformationChangeTestData.correctConstantRetryArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<DeviceInformationChangePlatformOperation>(),
              )),
            );
          });
        });

        group("When exponential retry policy is given", () {
          test("Then result is success", () async {
            final operation = MockDeviceInformationChangePlatformOperation();
            final retryPolicy = MockRetryPolicy();
            when(cache.createOperation(any)).thenReturn(operation);
            when(retryPolicy.toJson()).thenReturn(
              RetryPolicyTestData.exponentialRetryPolicyMap,
            );

            await sdk.operations.deviceInformationChange(
              operationId: CommonTestData.operationId,
              name: deviceName,
              fcmRegistrationToken: fcmToken,
              disablePushNotifications: false,
              retryPolicy: retryPolicy,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              DeviceInformationChangeTestData.correctExponentialRetryArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<DeviceInformationChangePlatformOperation>(),
              )),
            );
          });
        });
      });
    });
  });
}
