// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/error_test_data.dart';
import '../test_data/oob_operation_test_data.dart';
import '../test_data/out_of_band_payload_test_data.dart';
import '../test_data/request_headers_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache cache;
    const methodName = 'oobOperation';

    group("Given $methodName is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        cache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = cache;
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the outOfBandPayload is correct", () {
        test("Then result is success", () async {
          final channel = MethodChannelMock(
            method: methodName,
            result: CommonTestData.emptyResponse,
          );
          final requestHeaders = MockRequestHeaders();
          final operation = MockOutOfBandPlatformOperation();
          when(cache.createOperation(any)).thenReturn(operation);
          when(requestHeaders.toJson()).thenReturn(
            RequestHeadersTestData.requestHeadersMap,
          );

          await sdk.operations.outOfBandOperation(
            operationId: CommonTestData.operationId,
            subOperationId: CommonTestData.operationId,
            payload: OutOfBandPayloadTestData.payload,
            onRegistration: (registration) {},
            onAuthentication: (authentication) {},
            requestHeaders: requestHeaders,
            onOperationType: (type) {},
            onError: (error) {},
          );

          expect(
            channel.mappedArgs,
            OobOperationTestData.correctArguments,
          );
          verify(
            cache.createOperation(argThat(
              isA<OutOfBandPlatformOperation>(),
            )),
          );
        });
      });

      group("When the outOfBandPayload was already redeemed", () {
        test(
            "Then PlatformChannelException is thrown with OutOfBandOperationError (tokenAlreadyRedeemed) type",
            () async {
          final channel = MethodChannelMock(
            method: methodName,
            result: PlatformChannelException(
              error: OutOfBandOperationError.tokenAlreadyRedeemed(
                ErrorTestData.simpleErrorMap,
              ),
            ),
          );
          try {
            final requestHeaders = MockRequestHeaders();
            final operation = MockOutOfBandPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);
            when(requestHeaders.toJson()).thenReturn(
              RequestHeadersTestData.requestHeadersMap,
            );

            await sdk.operations.outOfBandOperation(
              operationId: CommonTestData.operationId,
              subOperationId: CommonTestData.operationId,
              payload: OutOfBandPayloadTestData.payload,
              onRegistration: (registration) {},
              onAuthentication: (authentication) {},
              requestHeaders: requestHeaders,
              onOperationType: (type) {},
              onError: (error) {},
            );
            fail('Exception expected');
          } on PlatformException catch (e) {
            expect(e.code, 'error');
          } catch (e) {
            fail('PlatformException expected');
          }

          expect(
            channel.mappedArgs,
            OobOperationTestData.correctArguments,
          );
          verify(
            cache.createOperation(argThat(
              isA<OutOfBandPlatformOperation>(),
            )),
          );
        });
      });
    });
  });
}
