// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/device_information_test_data.dart';
import '../test_data/oob_registration_test_data.dart';
import '../test_data/pin_policy_test_data.dart';
import '../test_data/request_headers_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache cache;
    late MethodChannelMock channel;
    const methodName = 'oobRegister';

    group("Given $methodName is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        cache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = cache;
        channel = MethodChannelMock(
          method: methodName,
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        group("When the required parameters are correct", () {
          test("Then result is success", () async {
            final authenticatorSelector = MockAuthenticatorSelector();
            final pinEnroller = MockPinEnroller();
            final biometricUserVerifier = MockBiometricUserVerifier();
            final devicePasscodeUserVerifier = MockDevicePasscodeUserVerifier();
            final fingerprintUserVerifier = MockFingerprintUserVerifier();
            final operation = MockOutOfBandRegistrationPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);
            when(pinEnroller.pinPolicy).thenReturn(
              PinPolicyTestData.pinPolicy,
            );

            await sdk.operations.outOfBandRegister(
              operationId: CommonTestData.operationId,
              authenticatorSelector: authenticatorSelector,
              pinEnroller: pinEnroller,
              biometricUserVerifier: biometricUserVerifier,
              devicePasscodeUserVerifier: devicePasscodeUserVerifier,
              fingerprintUserVerifier: fingerprintUserVerifier,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              OobRegistrationTestData.correctArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<OutOfBandRegistrationPlatformOperation>(),
              )),
            );
          });
        });

        group("When all parameters are correct", () {
          test("Then result is success", () async {
            final deviceInformation = MockDeviceInformation();
            final requestHeaders = MockRequestHeaders();
            final authenticatorSelector = MockAuthenticatorSelector();
            final pinEnroller = MockPinEnroller();
            final biometricUserVerifier = MockBiometricUserVerifier();
            final devicePasscodeUserVerifier = MockDevicePasscodeUserVerifier();
            final fingerprintUserVerifier = MockFingerprintUserVerifier();
            final operation = MockOutOfBandRegistrationPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);
            when(deviceInformation.toJson()).thenReturn(
              DeviceInformationTestData.deviceInformationMap,
            );
            when(pinEnroller.pinPolicy).thenReturn(
              PinPolicyTestData.pinPolicy,
            );
            when(requestHeaders.toJson()).thenReturn(
              RequestHeadersTestData.requestHeadersMap,
            );

            await sdk.operations.outOfBandRegister(
              operationId: CommonTestData.operationId,
              deviceInformation: deviceInformation,
              allowClass2AndroidSensors: true,
              allowDevicePasscodeAsFallback: true,
              invalidateOnNewOsBiometrics: true,
              requestHeaders: requestHeaders,
              authenticatorSelector: authenticatorSelector,
              pinEnroller: pinEnroller,
              biometricUserVerifier: biometricUserVerifier,
              devicePasscodeUserVerifier: devicePasscodeUserVerifier,
              fingerprintUserVerifier: fingerprintUserVerifier,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              OobRegistrationTestData.correctAllArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<OutOfBandRegistrationPlatformOperation>(),
              )),
            );
          });
        });
      });
    });
  });
}
